/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <aversive.h>
#include <aversive/queue.h>
#include <aversive/pgmspace.h>

#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <string.h>

#include <parse.h>

#include "parse_monitor.h"

struct monitor_reg_list xbee_monitor_list = LIST_HEAD_INITIALIZER();

static int8_t
parse_monitor(parse_pgm_token_hdr_t *tk, const char *buf, void *res)
{
	struct monitor_reg *m;
	uint8_t token_len = 0;
	char bufcopy[32];

	while (!isendoftoken(buf[token_len]) &&
	       token_len < (sizeof(bufcopy)-1)) {
		bufcopy[token_len] = buf[token_len];
		token_len++;
	}
	bufcopy[token_len] = 0;

	LIST_FOREACH(m, &xbee_monitor_list, next) {
		if (!strcmp_P(bufcopy, m->desc))
			break;
	}
	if (m == NULL) /* not found */
		return -1;

	/* store the address of object in structure */
	if (res)
		*(struct monitor_reg **)res = m;

	return token_len;
}

static int8_t
complete_get_nb_monitor(parse_pgm_token_hdr_t *tk)
{
	struct monitor_reg *m;
	int8_t i = 0;

	LIST_FOREACH(m, &xbee_monitor_list, next) {
		i++;
	}
	return i;
}

static int8_t
complete_get_elt_monitor(parse_pgm_token_hdr_t *tk, int8_t idx,
			  char *dstbuf, uint8_t size)
{
	struct monitor_reg *m;
	int8_t i = 0, len;

	LIST_FOREACH(m, &xbee_monitor_list, next) {
		if (i == idx)
			break;
		i++;
	}
	if (m == NULL)
		return -1;

	len = snprintf(dstbuf, size, "%S", m->desc);
	if (len < 0 || len >= size)
		return -1;

	return 0;
}


static int8_t
help_monitor(parse_pgm_token_hdr_t *tk, char *dstbuf,
	      uint8_t size)
{
	snprintf(dstbuf, size, "Monitor-register");
	return 0;
}

struct token_ops token_monitor_ops = {
	.parse = parse_monitor,
	.complete_get_nb = complete_get_nb_monitor,
	.complete_get_elt = complete_get_elt_monitor,
	.get_help = help_monitor,
};
